@@ -25,11 +25,16 @@ ignored_files = {
   "testOOM.c": "out of memory tester",
   "testOOMlib.h": "out of memory tester",
   "testOOMlib.c": "out of memory tester",
+  "pattern.c": "not integrated yet",
+  "pattern.h": "not integrated yet",
 }
 
 ignored_words = {
   "WINAPI": (0, "Windows keyword"),
   "LIBXML_DLL_IMPORT": (0, "Special macro to flag external keywords"),
+  "XMLPUBVAR": (0, "Special macro for extern vars for win32"),
+  "XMLPUBFUN": (0, "Special macro for extern funcs for win32"),
+  "XMLCALL": (0, "Special macro for win32 calls"),
   "__declspec": (3, "Windows keyword"),
   "ATTRIBUTE_UNUSED": (0, "macro keyword"),
   "LIBEXSLT_PUBLIC": (0, "macro keyword"),
@@ -1322,7 +1327,11 @@ class docBuilder:
 	 if id.info != None:
 	     info = id.info
 	     if info[0] != None and info[0] != '':
-		 output.write(" value='%s'" % info[0]);
+	         try:
+		     val = eval(info[0])
+		 except:
+		     val = info[0]
+		 output.write(" value='%s'" % (val));
 	     if info[2] != None and info[2] != '':
 		 output.write(" type='%s'" % info[2]);
 	     if info[1] != None and info[1] != '':
