@@ -18,6 +18,7 @@
 #ifndef __XML_HASH_H__
 #define __XML_HASH_H__
 
+#include <libxml/xmlversion.h>
 #include <libxml/parser.h>
 
 #ifdef __cplusplus
@@ -25,7 +26,7 @@ extern "C" {
 #endif
 
 /*
- * The hash table
+ * The hash table.
  */
 typedef struct _xmlHashTable xmlHashTable;
 typedef xmlHashTable *xmlHashTablePtr;
@@ -38,7 +39,7 @@ typedef xmlHashTable *xmlHashTablePtr;
  * @payload:  the data in the hash
  * @name:  the name associated
  *
- * Callback to free data from a hash
+ * Callback to free data from a hash.
  */
 typedef void (*xmlHashDeallocator)(void *payload, xmlChar *name);
 /**
@@ -46,9 +47,9 @@ typedef void (*xmlHashDeallocator)(void *payload, xmlChar *name);
  * @payload:  the data in the hash
  * @name:  the name associated
  *
- * Callback to copy data from a hash
+ * Callback to copy data from a hash.
  *
- * Returns a copy of the data or NULL in case of error
+ * Returns a copy of the data or NULL in case of error.
  */
 typedef void *(*xmlHashCopier)(void *payload, xmlChar *name);
 /**
@@ -57,7 +58,7 @@ typedef void *(*xmlHashCopier)(void *payload, xmlChar *name);
  * @data:  extra scannner data
  * @name:  the name associated
  *
- * Callback when scanning data in a hash with the simple scanner
+ * Callback when scanning data in a hash with the simple scanner.
  */
 typedef void (*xmlHashScanner)(void *payload, void *data, xmlChar *name);
 /**
@@ -68,44 +69,52 @@ typedef void (*xmlHashScanner)(void *payload, void *data, xmlChar *name);
  * @name2:  the second name associated
  * @name3:  the third name associated
  *
- * Callback when scanning data in a hash with the full scanner
+ * Callback when scanning data in a hash with the full scanner.
  */
 typedef void (*xmlHashScannerFull)(void *payload, void *data,
 				   const xmlChar *name, const xmlChar *name2,
 				   const xmlChar *name3);
 
 /*
- * Constructor and destructor
+ * Constructor and destructor.
  */
-xmlHashTablePtr		xmlHashCreate	(int size);
-void			xmlHashFree	(xmlHashTablePtr table,
+XMLPUBFUN xmlHashTablePtr XMLCALL
+			xmlHashCreate	(int size);
+XMLPUBFUN void XMLCALL			
+			xmlHashFree	(xmlHashTablePtr table,
 					 xmlHashDeallocator f);
 
 /*
- * Add a new entry to the hash table
+ * Add a new entry to the hash table.
  */
-int			xmlHashAddEntry	(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashAddEntry	(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         void *userdata);
-int			xmlHashUpdateEntry(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashUpdateEntry(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         void *userdata,
 					 xmlHashDeallocator f);
-int			xmlHashAddEntry2(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL		    
+			xmlHashAddEntry2(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         const xmlChar *name2,
 		                         void *userdata);
-int			xmlHashUpdateEntry2(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashUpdateEntry2(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         const xmlChar *name2,
 		                         void *userdata,
 					 xmlHashDeallocator f);
-int			xmlHashAddEntry3(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashAddEntry3(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         const xmlChar *name2,
 		                         const xmlChar *name3,
 		                         void *userdata);
-int			xmlHashUpdateEntry3(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashUpdateEntry3(xmlHashTablePtr table,
 		                         const xmlChar *name,
 		                         const xmlChar *name2,
 		                         const xmlChar *name3,
@@ -113,48 +122,78 @@ int			xmlHashUpdateEntry3(xmlHashTablePtr table,
 					 xmlHashDeallocator f);
 
 /*
- * Remove an entry from the hash table
+ * Remove an entry from the hash table.
  */
-int     xmlHashRemoveEntry(xmlHashTablePtr table, const xmlChar *name,
+XMLPUBFUN int XMLCALL     
+			xmlHashRemoveEntry(xmlHashTablePtr table, const xmlChar *name,
                            xmlHashDeallocator f);
-int     xmlHashRemoveEntry2(xmlHashTablePtr table, const xmlChar *name,
+XMLPUBFUN int XMLCALL     
+			xmlHashRemoveEntry2(xmlHashTablePtr table, const xmlChar *name,
                             const xmlChar *name2, xmlHashDeallocator f);
-int     xmlHashRemoveEntry3(xmlHashTablePtr table, const xmlChar *name,
+XMLPUBFUN int  XMLCALL    
+			xmlHashRemoveEntry3(xmlHashTablePtr table, const xmlChar *name,
                             const xmlChar *name2, const xmlChar *name3,
                             xmlHashDeallocator f);
 
 /*
- * Retrieve the userdata
+ * Retrieve the userdata.
  */
-void *			xmlHashLookup	(xmlHashTablePtr table,
+XMLPUBFUN void * XMLCALL			
+			xmlHashLookup	(xmlHashTablePtr table,
 					 const xmlChar *name);
-void *			xmlHashLookup2	(xmlHashTablePtr table,
+XMLPUBFUN void * XMLCALL			
+			xmlHashLookup2	(xmlHashTablePtr table,
 					 const xmlChar *name,
 					 const xmlChar *name2);
-void *			xmlHashLookup3	(xmlHashTablePtr table,
+XMLPUBFUN void * XMLCALL			
+			xmlHashLookup3	(xmlHashTablePtr table,
 					 const xmlChar *name,
 					 const xmlChar *name2,
 					 const xmlChar *name3);
+XMLPUBFUN void * XMLCALL			
+			xmlHashQLookup	(xmlHashTablePtr table,
+					 const xmlChar *name,
+					 const xmlChar *prefix);
+XMLPUBFUN void * XMLCALL			
+			xmlHashQLookup2	(xmlHashTablePtr table,
+					 const xmlChar *name,
+					 const xmlChar *prefix,
+					 const xmlChar *name2,
+					 const xmlChar *prefix2);
+XMLPUBFUN void * XMLCALL			
+			xmlHashQLookup3	(xmlHashTablePtr table,
+					 const xmlChar *name,
+					 const xmlChar *prefix,
+					 const xmlChar *name2,
+					 const xmlChar *prefix2,
+					 const xmlChar *name3,
+					 const xmlChar *prefix3);
 
 /*
- * Helpers
+ * Helpers.
  */
-xmlHashTablePtr		xmlHashCopy	(xmlHashTablePtr table,
+XMLPUBFUN xmlHashTablePtr XMLCALL		
+			xmlHashCopy	(xmlHashTablePtr table,
 					 xmlHashCopier f);
-int			xmlHashSize	(xmlHashTablePtr table);
-void			xmlHashScan	(xmlHashTablePtr table,
+XMLPUBFUN int XMLCALL			
+			xmlHashSize	(xmlHashTablePtr table);
+XMLPUBFUN void XMLCALL			
+			xmlHashScan	(xmlHashTablePtr table,
 					 xmlHashScanner f,
 					 void *data);
-void			xmlHashScan3	(xmlHashTablePtr table,
+XMLPUBFUN void XMLCALL			
+			xmlHashScan3	(xmlHashTablePtr table,
 					 const xmlChar *name,
 					 const xmlChar *name2,
 					 const xmlChar *name3,
 					 xmlHashScanner f,
 					 void *data);
-void			xmlHashScanFull	(xmlHashTablePtr table,
+XMLPUBFUN void XMLCALL			
+			xmlHashScanFull	(xmlHashTablePtr table,
 					 xmlHashScannerFull f,
 					 void *data);
-void			xmlHashScanFull3(xmlHashTablePtr table,
+XMLPUBFUN void XMLCALL			
+			xmlHashScanFull3(xmlHashTablePtr table,
 					 const xmlChar *name,
 					 const xmlChar *name2,
 					 const xmlChar *name3,
