@@ -300,6 +300,27 @@ def skip_function(name):
         return 1
 #    if name[0:11] == "xmlXPathNew":
 #        return 1
+    # the next function is defined in libxml.c
+    if name == "xmlRelaxNGFreeValidCtxt":
+        return 1
+#
+# Those are skipped because the Const version is used of the bindings
+# instead.
+#
+    if name == "xmlTextReaderBaseUri":
+        return 1
+    if name == "xmlTextReaderLocalName":
+        return 1
+    if name == "xmlTextReaderName":
+        return 1
+    if name == "xmlTextReaderNamespaceUri":
+        return 1
+    if name == "xmlTextReaderPrefix":
+        return 1
+    if name == "xmlTextReaderXmlLang":
+        return 1
+    if name == "xmlTextReaderValue":
+        return 1
     return 0
 
 def print_function_wrapper(name, output, export, include):
@@ -362,8 +383,8 @@ def print_function_wrapper(name, output, export, include):
 	    if args[1][1] == "char *" or args[1][1] == "xmlChar *":
 		c_call = "\n    if (%s->%s != NULL) xmlFree(%s->%s);\n" % (
 		                 args[0][0], args[1][0], args[0][0], args[1][0])
-		c_call = c_call + "    %s->%s = xmlStrdup((const xmlChar *)%s);\n" % (args[0][0],
-		                 args[1][0], args[1][0])
+		c_call = c_call + "    %s->%s = (%s)xmlStrdup((const xmlChar *)%s);\n" % (args[0][0],
+		                 args[1][0], args[1][1], args[1][0])
 	    else:
 		c_call = "\n    %s->%s = %s;\n" % (args[0][0], args[1][0],
 						   args[1][0])
@@ -550,6 +571,7 @@ def buildStubs():
 #    wrapper.write("#include \"config.h\"\n")
     wrapper.write("#include <libxml/xmlversion.h>\n")
     wrapper.write("#include <libxml/tree.h>\n")
+    wrapper.write("#include <libxml/xmlschemastypes.h>\n")
     wrapper.write("#include \"libxml_wrap.h\"\n")
     wrapper.write("#include \"libxml2-py.h\"\n\n")
     for function in functions.keys():
@@ -714,8 +736,12 @@ def nameFixup(name, classe, type, file):
         func = "regexp" + name[6:]
     elif name[0:20] == "xmlTextReaderLocator" and file == "xmlreader":
         func = name[20:]
+    elif name[0:18] == "xmlTextReaderConst" and file == "xmlreader":
+        func = name[18:]
     elif name[0:13] == "xmlTextReader" and file == "xmlreader":
         func = name[13:]
+    elif name[0:12] == "xmlReaderNew" and file == "xmlreader":
+        func = name[9:]
     elif name[0:11] == "xmlACatalog":
         func = name[11:]
         func = string.lower(func[0:1]) + func[1:]
