@@ -128,6 +128,7 @@ static int repeat = 0;
 static int insert = 0;
 #ifdef  LIBXML_HTML_ENABLED
 static int html = 0;
+static int xmlout = 0;
 #endif
 static int htmlout = 0;
 #ifdef LIBXML_PUSH_ENABLED
@@ -153,6 +154,7 @@ static int nocatalogs = 0;
 #endif
 #ifdef LIBXML_READER_ENABLED
 static int stream = 0;
+static int walker = 0;
 #endif /* LIBXML_READER_ENABLED */
 static int chkregister = 0;
 #ifdef LIBXML_SAX1_ENABLED
@@ -603,7 +605,7 @@ static void processNode(xmlTextReaderPtr reader) {
 
     name = xmlTextReaderConstName(reader);
     if (name == NULL)
-	name = xmlStrdup(BAD_CAST "--");
+	name = BAD_CAST "--";
 
     value = xmlTextReaderConstValue(reader);
 
@@ -638,12 +640,12 @@ static void streamFile(char *filename) {
 	if (base == (void *) MAP_FAILED)
 	    return;
 
-	input = xmlParserInputBufferCreateStatic((char *) base, info.st_size,
-	                                      XML_CHAR_ENCODING_NONE);
-	reader = xmlNewTextReader(input, filename);
+	reader = xmlReaderForMemory(base, info.st_size, filename,
+	                            NULL, options);
     } else
 #endif
-	reader = xmlNewTextReaderFilename(filename);
+	reader = xmlReaderForFile(filename, NULL, options);
+
 
     if (reader != NULL) {
 #ifdef LIBXML_VALID_ENABLED
@@ -734,6 +736,35 @@ static void streamFile(char *filename) {
     }
 #endif
 }
+
+static void walkDoc(xmlDocPtr doc) {
+    xmlTextReaderPtr reader;
+    int ret;
+
+    reader = xmlReaderWalker(doc);
+    if (reader != NULL) {
+	if ((timing) && (!repeat)) {
+	    startTimer();
+	}
+	ret = xmlTextReaderRead(reader);
+	while (ret == 1) {
+	    if (debug)
+		processNode(reader);
+	    ret = xmlTextReaderRead(reader);
+	}
+	if ((timing) && (!repeat)) {
+	    endTimer("walking through the doc");
+	}
+	xmlFreeTextReader(reader);
+	if (ret != 0) {
+	    printf("failed to walk through the doc\n");
+	    progresult = 1;
+	}
+    } else {
+	fprintf(stderr, "Failed to crate a reader from the document\n");
+	progresult = 1;
+    }
+}
 #endif /* LIBXML_READER_ENABLED */
 
 /************************************************************************
@@ -779,7 +810,7 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
             res = fread(chars, 1, 4, f);
             if (res > 0) {
                 ctxt = htmlCreatePushParserCtxt(NULL, NULL,
-                            chars, res, filename, 0);
+                            chars, res, filename, XML_CHAR_ENCODING_NONE);
                 while ((res = fread(chars, 1, size, f)) > 0) {
                     htmlParseChunk(ctxt, chars, res, 0);
                 }
@@ -1006,6 +1037,11 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	}    
     }else
 #endif /* LIBXML_VALID_ENABLED */
+#ifdef LIBXML_READER_ENABLED
+    if (walker) {
+        walkDoc(doc);
+    }
+#endif /* LIBXML_READER_ENABLED */
 #ifdef LIBXML_OUTPUT_ENABLED
     if (noout == 0) {
 	/*
@@ -1017,6 +1053,45 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	    if ((timing) && (!repeat)) {
 		startTimer();
 	    }
+#ifdef LIBXML_VALID_ENABLED
+            if ((html) && (!xmlout)) {
+		if (compress) {
+		    htmlSaveFile(output ? output : "-", doc);
+		}
+		else if (encoding != NULL) {
+		    if ( format ) {
+			htmlSaveFileFormat(output ? output : "-", doc, encoding, 1);
+		    }
+		    else {
+			htmlSaveFileFormat(output ? output : "-", doc, encoding, 0);
+		    }
+		}
+		else if (format) {
+		    htmlSaveFileFormat(output ? output : "-", doc, NULL, 1);
+		}
+		else {
+		    FILE *out;
+		    if (output == NULL)
+			out = stdout;
+		    else {
+			out = fopen(output,"wb");
+		    }
+		    if (out != NULL) {
+			if (htmlDocDump(out, doc) < 0)
+			    progresult = 6;
+
+			if (output != NULL)
+			    fclose(out);
+		    } else {
+			fprintf(stderr, "failed to open %s\n", output);
+			progresult = 6;
+		    }
+		}
+		if ((timing) && (!repeat)) {
+		    endTimer("Saving");
+		}
+	    } else
+#endif
 #ifdef HAVE_SYS_MMAN_H
 	    if (memory) {
 		xmlChar *result;
@@ -1355,6 +1430,7 @@ static void usage(const char *name) {
 #endif /* LIBXML_OUTPUT_ENABLED */
 #ifdef LIBXML_HTML_ENABLED
     printf("\t--html : use the HTML parser\n");
+    printf("\t--xmlout : force to use the XML serializer when using --html\n");
 #endif
 #ifdef LIBXML_PUSH_ENABLED
     printf("\t--push : use the push mode of the parser\n");
@@ -1386,6 +1462,7 @@ static void usage(const char *name) {
     printf("\t--dtdattr : loaddtd + populate the tree with inherited attributes \n");
 #ifdef LIBXML_READER_ENABLED
     printf("\t--stream : use the streaming interface to process very large files\n");
+    printf("\t--walker : create a reader and walk though the resulting doc\n");
 #endif /* LIBXML_READER_ENABLED */
     printf("\t--chkregister : verify the node registration code\n");
 #ifdef LIBXML_SCHEMAS_ENABLED
@@ -1485,6 +1562,10 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--html"))) {
 	    html++;
         }
+	else if ((!strcmp(argv[i], "-xmlout")) ||
+	         (!strcmp(argv[i], "--xmlout"))) {
+	    xmlout++;
+        }
 #endif /* LIBXML_HTML_ENABLED */
 	else if ((!strcmp(argv[i], "-loaddtd")) ||
 	         (!strcmp(argv[i], "--loaddtd"))) {
@@ -1553,7 +1634,7 @@ main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-xinclude")) ||
 	         (!strcmp(argv[i], "--xinclude"))) {
 	    xinclude++;
-	    /* options |= XML_PARSE_XINCLUDE; */
+	    options |= XML_PARSE_XINCLUDE;
 	}
 #endif
 #ifdef LIBXML_OUTPUT_ENABLED
@@ -1620,6 +1701,11 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--stream"))) {
 	     stream++;
 	}
+	else if ((!strcmp(argv[i], "-walker")) ||
+	         (!strcmp(argv[i], "--walker"))) {
+	     walker++;
+             noout++;
+	}
 #endif /* LIBXML_READER_ENABLED */
 #ifdef LIBXML_SAX1_ENABLED
 	else if ((!strcmp(argv[i], "-sax1")) ||
@@ -1645,7 +1731,7 @@ main(int argc, char **argv) {
 #endif
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
-	    xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
+	    options |= XML_PARSE_NONET;
 	} else {
 	    fprintf(stderr, "Unknown option %s\n", argv[i]);
 	    usage(argv[0]);
@@ -1801,9 +1887,6 @@ main(int argc, char **argv) {
 	if ((argv[i][0] != '-') || (strcmp(argv[i], "-") == 0)) {
 	    if (repeat) {
 		xmlParserCtxtPtr ctxt = NULL;
-#ifdef LIBXML_READER_ENABLED
-                xmlTextReaderPtr reader = NULL;
-#endif /* LIBXML_READER_ENABLED */
 
 		for (acount = 0;acount < repeat;acount++) {
 #ifdef LIBXML_READER_ENABLED
